@@ -40,6 +40,8 @@ from shutil import copy as copyfile
 import datetime as dt
 import gettext
 
+import itertools
+
 DB_FILE = 'hamster.db'
 
 class Storage(storage.Storage):
@@ -78,16 +80,76 @@ class Storage(storage.Storage):
         self.__setup.im_func.complete = True
         self.run_fixtures()
 
-        self.config = GconfStore()
-
-        runtime.dispatcher.add_handler('gconf_on_day_start_changed', self.__on_day_start_changed)
-        self.day_start = self.config.get_day_start()
 
 
     __setup.complete = False
     
-    def __on_day_start_changed(self, event, new_minutes):
-        self.day_start = self.config.get_day_start()
+    #tags, here we come!
+    def __get_tags(self, autocomplete = None):
+        query = "select * from tags"
+        if autocomplete:
+            query += " where autocomplete='true'"
+        
+        query += " order by name"
+        return self.fetchall(query)
+        
+    def __get_tag_ids(self, tags):
+        """look up tags by their name. create if not found"""
+        
+        # filter descriptions out, just in case they have wandered in here
+        tags = [tag for tag in tags if tag.startswith("!") == False and len(tag.split(" ")) < 3]
+        
+        db_tags = self.fetchall("select * from tags where name in (%s)"
+                                            % ",".join(["?"] * len(tags)), tags) # bit of magic here - using sqlites bind variables
+        
+        changes = False
+        
+        # check if any of tags needs ressurection
+        set_complete = [str(tag["id"]) for tag in db_tags if tag["autocomplete"] == "false"]
+        if set_complete:
+            changes = True
+            self.execute("update tags set autocomplete='true' where id in (%s)" % ", ".join(set_complete))
+        
+        
+        found_tags = [tag["name"] for tag in db_tags]
+        
+        add = set(tags) - set(found_tags)
+        if add:
+            statement = "insert into tags(name) values(?)"
+
+            self.execute([statement] * len(add), [(tag,) for tag in add])
+
+            return self.__get_tag_ids(tags)[0], True # all done, recurse
+        else:
+            return db_tags, changes
+
+    def __update_autocomplete_tags(self, tags):
+        tags = [tag.strip() for tag in tags.split(",") if tag.strip()]  # split by comma
+
+        #first we will create new ones
+        tags, changes = self.__get_tag_ids(tags)
+        tags = [tag["id"] for tag in tags]
+        
+        #now we will find which ones are gone from the list
+        query = """
+                    SELECT b.id as id, count(a.fact_id) as occurences
+                      FROM tags b
+                 LEFT JOIN fact_tags a on a.tag_id = b.id
+                     WHERE b.id not in (%s)
+                  GROUP BY b.id
+                """ % ",".join(["?"] * len(tags)) # bit of magic here - using sqlites bind variables
+
+        gone = self.fetchall(query, tags)
+
+        to_delete = [str(tag["id"]) for tag in gone if tag["occurences"] == 0]
+        to_uncomplete = [str(tag["id"]) for tag in gone if tag["occurences"] > 0]
+
+        if to_delete:
+            self.execute("delete from tags where id in (%s)" % ", ".join(to_delete))
+
+        if to_uncomplete:        
+            self.execute("update tags set autocomplete='false' where id in (%s)" % ", ".join(to_uncomplete))
+
 
     def __get_category_list(self):
         return self.fetchall("SELECT * FROM categories ORDER BY category_order")
@@ -223,18 +285,35 @@ class Storage(storage.Storage):
         return None
 
     def __get_fact(self, id):
-        query = """SELECT a.id AS id,
+        query = """
+                   SELECT a.id AS id,
                           a.start_time AS start_time,
                           a.end_time AS end_time,
                           a.description as description,
                           b.name AS name, b.id as activity_id,
-                          coalesce(c.name, ?) as category, coalesce(c.id, -1) as category_id
+                          coalesce(c.name, ?) as category, coalesce(c.id, -1) as category_id,
+                          e.name as tags
                      FROM facts a
                 LEFT JOIN activities b ON a.activity_id = b.id
-                LEFT JOIN categories c on b.category_id = c.id
+                LEFT JOIN categories c ON b.category_id = c.id
+                LEFT JOIN fact_tags d ON d.fact_id = a.id
+                LEFT JOIN tags e ON e.id = d.tag_id
                     WHERE a.id = ? 
         """
-        return self.fetchone(query, (_("Unsorted"), id))
+
+        return self.__group_tags(self.fetchall(query, (_("Unsorted"), id)))[0]
+
+    def __group_tags(self, facts):
+        """put the fact back together and move all the unique tags to an array"""
+        if not facts: return facts  #be it None or whatever
+        
+        grouped_facts = []
+        for fact_id, fact_tags in itertools.groupby(facts, lambda f: f["id"]):
+            fact_tags = list(fact_tags)
+            grouped_fact = dict(fact_tags[0]) #first is as good as the last one
+            grouped_fact["tags"] = [ft["tags"] for ft in fact_tags if ft["tags"]]
+            grouped_facts.append(grouped_fact)
+        return grouped_facts
 
     def __get_last_activity(self):
         query = """
@@ -243,26 +322,32 @@ class Storage(storage.Storage):
                           a.end_time AS end_time,
                           a.description as description,
                           b.name AS name, b.id as activity_id,
-                          coalesce(c.name, ?) as category, coalesce(c.id, -1) as category_id
+                          coalesce(c.name, ?) as category, coalesce(c.id, -1) as category_id,
+                          e.name as tags
                      FROM facts a
                 LEFT JOIN activities b ON a.activity_id = b.id
-                LEFT JOIN categories c on b.category_id = c.id
+                LEFT JOIN categories c ON b.category_id = c.id
+                LEFT JOIN fact_tags d ON d.fact_id = a.id
+                LEFT JOIN tags e ON e.id = d.tag_id
                     WHERE date(a.start_time) = ?
                  ORDER BY a.start_time desc
-                    LIMIT 1
+                    LIMIT 20
         """
-        last = self.fetchone(query, (_("Unsorted"), dt.date.today()))
+        last = self.fetchall(query, (_("Unsorted"), dt.date.today()))        
         if not last:
             #try yesterday if there is nothing today
-            last = self.fetchone(query,
+            last = self.fetchall(query,
                                  (# unsorted category
                                   _("Unsorted"),
                                   dt.date.today() - dt.timedelta(days=1)))
 
+        last = self.__group_tags(last)[0]
+
         if last and last["end_time"]: #will consider as last only if it is going on
            last = None
         
         return last
+
     def __touch_fact(self, fact, end_time):
         # tasks under one minute do not count
         if end_time - fact['start_time'] < datetime.timedelta(minutes = 1):
@@ -305,7 +390,13 @@ class Storage(storage.Storage):
                 #we are in middle of a fact - truncate it to our start
                 self.execute("UPDATE facts SET end_time=? WHERE id=?",
                              (start_time, fact["id"]))
-                end_time = fact["end_time"]
+                
+                # hamster is second-aware, but the edit dialog naturally is not
+                # so when an ongoing task is being edited, the seconds get truncated
+                # and the start time will be before previous task's end time.
+                # so set our end time only if it is not about seconds
+                if fact["end_time"].replace(second = 0) > start_time:
+                    end_time = fact["end_time"]
             else: #otherwise we have found a task that is after us
                 end_time = fact["start_time"]
 
@@ -316,7 +407,7 @@ class Storage(storage.Storage):
         make room for new fact"""
 
         # this function is destructive - can't go with a wildcard
-        if not end_time or not start_time: #this function is destructive
+        if not end_time or not start_time:
             return
         
         # activities that we are overlapping.
@@ -346,11 +437,17 @@ class Storage(storage.Storage):
                                    SET end_time = ?
                                  WHERE id = ?""", (start_time, fact["id"]))
                 fact_name = fact["name"]
-                self.__add_fact(fact["name"],
-                                end_time,
-                                fact["end_time"],
-                                fact["category"],
-                                fact["description"])
+                new_fact = self.__add_fact(fact["name"],
+                                           "", # will create tags in the next step
+                                           end_time,
+                                           fact["end_time"],
+                                           fact["category"],
+                                           fact["description"])
+                tag_update = """INSERT INTO fact_tags(fact_id, tag_id)
+                                     SELECT ?, tag_id
+                                       FROM fact_tags
+                                      WHERE fact_id = ?"""
+                self.execute(tag_update, (new_fact["id"], fact["id"])) #clone tags
 
             #eliminate
             elif fact["end_time"] and \
@@ -372,14 +469,23 @@ class Storage(storage.Storage):
                              (start_time, fact["id"]))
 
 
-    def __add_fact(self, activity_name, start_time = None,
+    def __add_fact(self, activity_name, tags, start_time = None,
                      end_time = None, category_name = None, description = None):
+        
         activity = stuff.parse_activity_input(activity_name)
         
+        tags = [tag.strip() for tag in tags.split(",") if tag.strip()]  # split by comma
+        descriptions = [tag for tag in tags if len(tag.split(" ")) > 2 or tag.startswith("!")]  #extract description
+        tags = list(set(tags) - set(descriptions)) #remove any found descriptions from tag list
+        
+        # TODO - untangle descriptions - allow just one place where to enter them
+        activity.description = ", ".join(descriptions) # somebody will file bug on "why tags can't be seven words"
+        tags = self.get_tag_ids(tags) #this will create any missing tags too
+        
         if category_name:
             activity.category_name = category_name
         if description:
-            activity.description = description
+            activity.description = description #override
         
         start_time = activity.start_time or start_time or datetime.datetime.now()
         
@@ -413,12 +519,12 @@ class Storage(storage.Storage):
 
             if last_activity and last_activity['start_time'] < start_time:
                 #if this is the same, ongoing activity, then there is no need to create another one
-                if not activity.description \
+                if not tags and not activity.description \
                    and last_activity['activity_id'] == activity_id:
                     return last_activity
                 
                 #if duration is less than a minute - it must have been a mistake
-                if not activity.description \
+                if not tags and not activity.description \
                    and not last_activity["description"] \
                    and 60 >= (start_time - last_activity['start_time']).seconds >= 0:
                     self.__remove_fact(last_activity['id'])
@@ -449,6 +555,11 @@ class Storage(storage.Storage):
 
         fact_id = self.fetchone("select max(id) as max_id from facts")['max_id']
         
+        #now link tags
+        insert = ["insert into fact_tags(fact_id, tag_id) values(?, ?)"] * len(tags)
+        params = [(fact_id, tag["id"]) for tag in tags]
+        self.execute(insert, params)
+        
         return self.__get_fact(fact_id)
 
 
@@ -459,10 +570,13 @@ class Storage(storage.Storage):
                           a.end_time AS end_time,
                           a.description as description,
                           b.name AS name, b.id as activity_id,
-                          coalesce(c.name, ?) as category, coalesce(c.id, -1) as category_id
+                          coalesce(c.name, ?) as category, coalesce(c.id, -1) as category_id,
+                          e.name as tags
                      FROM facts a
                 LEFT JOIN activities b ON a.activity_id = b.id
                 LEFT JOIN categories c ON b.category_id = c.id
+                LEFT JOIN fact_tags d ON d.fact_id = a.id
+                LEFT JOIN tags e ON e.id = d.tag_id
                     WHERE (a.end_time >= ? OR a.end_time IS NULL) AND a.start_time <= ?
         """
         
@@ -474,14 +588,19 @@ class Storage(storage.Storage):
         query += " ORDER BY a.start_time"
         end_date = end_date or date
 
-        #FIXME: add preference to set that
-        split_time = self.day_start
+        from configuration import GconfStore
+        day_start = GconfStore().get_day_start()
+        
+        split_time = day_start
         datetime_from = dt.datetime.combine(date, split_time)
         datetime_to = dt.datetime.combine(end_date, split_time) + dt.timedelta(days = 1)
         
         facts = self.fetchall(query, (_("Unsorted"), datetime_from, datetime_to))
+        
+        #first let's put all tags in an array
+        facts = self.__group_tags(facts)
+        
         res = []
-
         for fact in facts:
             # heuristics to assign tasks to proper days
 
@@ -519,19 +638,9 @@ class Storage(storage.Storage):
                 # due to spanning we've jumped outside of given period
                 continue
 
-            f = dict(
-                id = fact["id"],
-                start_time = fact["start_time"],
-                date = fact_date,
-                delta = fact_end_time - fact["start_time"],
-                end_time = fact["end_time"],
-                description = fact["description"],
-                name = fact["name"],
-                activity_id = fact["activity_id"],
-                category = fact["category"],
-                category_id = fact["category_id"]
-            )
-            res.append(f)
+            fact["date"] = fact_date
+            fact["delta"] = fact_end_time - fact["start_time"]
+            res.append(fact)
 
         return res
 
@@ -548,11 +657,9 @@ class Storage(storage.Storage):
         return self.fetchall(query, (_("Unsorted"), ))
 
     def __remove_fact(self, fact_id):
-        query = """
-                   DELETE FROM facts
-                         WHERE id = ?
-        """
-        self.execute(query, (fact_id,))
+        statements = ["DELETE FROM fact_tags where fact_id = ?",
+                      "DELETE FROM facts where id = ?"]
+        self.execute(statements, [(fact_id,)] * 2)
 
     def __get_activities(self, category_id = None):
         """returns list of activities, if category is specified, order by name
@@ -746,7 +853,7 @@ class Storage(storage.Storage):
         
         """upgrade DB to hamster version"""
         version = self.fetchone("SELECT version FROM version")["version"]
-        current_version = 5
+        current_version = 6
 
         if version < 2:
             """moving from fact_date, fact_time to start_time, end_time"""
@@ -911,14 +1018,26 @@ class Storage(storage.Storage):
         if version < 5:
             self.execute("ALTER TABLE facts add column description varchar2")
 
+        if version < 6:
+            # facts table could use an index
+            self.execute("CREATE INDEX idx_facts_start_end ON facts(start_time, end_time)")
+            self.execute("CREATE INDEX idx_facts_start_end_activity ON facts(start_time, end_time, activity_id)")
+
+            # adding tags
+            self.execute("""CREATE TABLE tags (id INTEGER PRIMARY KEY AUTOINCREMENT NOT NULL,
+                                               name TEXT NOT NULL,
+                                               autocomplete BOOL DEFAULT true)""")
+            self.execute("CREATE INDEX idx_tags_name ON tags(name)")
+
+            self.execute("CREATE TABLE fact_tags(fact_id integer, tag_id integer)")
+            self.execute("CREATE INDEX idx_fact_tags_fact ON fact_tags(fact_id)")
+            self.execute("CREATE INDEX idx_fact_tags_tag ON fact_tags(tag_id)")
 
         # at the happy end, update version number 
         if version < current_version:
             #lock down current version
             self.execute("UPDATE version SET version = %d" % current_version) 
         
-
-        
         """we start with an empty database and then populate with default
            values. This way defaults can be localized!"""
         
