@@ -17,15 +17,19 @@
 # You should have received a copy of the GNU General Public License
 # along with Project Hamster.  If not, see <http://www.gnu.org/licenses/>.
 
-
+import logging
 import pygtk
 pygtk.require('2.0')
 
 import os
 import gtk
 
-from hamster import dispatcher, storage, SHARED_DATA_DIR, stuff
-from hamster.Configuration import GconfStore
+import dispatcher, storage, stuff
+
+import datetime as dt
+import widgets
+
+from configuration import GconfStore, runtime
 
 def get_prev(selection, model):
     (model, iter) = selection.get_selected()
@@ -46,7 +50,7 @@ class CategoryStore(gtk.ListStore):
         """ Loads activity list from database, ordered by
             activity_order """
 
-        category_list = storage.get_category_list()
+        category_list = runtime.storage.get_category_list()
 
         for category in category_list:
             self.append([category['id'],
@@ -67,10 +71,10 @@ class ActivityStore(gtk.ListStore):
             
         self.clear()
 
-        if category_id == None:
+        if category_id is None:
             return
         
-        activity_list = storage.get_activities(category_id)
+        activity_list = runtime.storage.get_activities(category_id)
 
         for activity in activity_list:
             self.append([activity['id'],
@@ -92,7 +96,7 @@ class PreferencesEditor:
     def __init__(self, parent = None):
         self.parent = parent
         self._gui = stuff.load_ui_file("preferences.ui")
-        self.config = GconfStore.get_instance()
+        self.config = GconfStore()
         self.window = self.get_widget('preferences_window')
 
 
@@ -138,9 +142,12 @@ class PreferencesEditor:
         selection = self.category_tree.get_selection()
         selection.connect('changed', self.category_changed_cb, self.category_store)
 
-        self.load_config()
+        self.day_start = widgets.TimeInput(dt.time(5,30))
+        self.get_widget("day_start_placeholder").add(self.day_start)
+        self.day_start.connect("time-entered", self.on_day_start_changed)
 
-        self._gui.connect_signals(self)
+
+        self.load_config()
 
         # Allow enable drag and drop of rows including row move
         self.activity_tree.enable_model_drag_source( gtk.gdk.BUTTON1_MASK,
@@ -173,14 +180,32 @@ class PreferencesEditor:
         except:
             self.get_widget("notification_preference_frame").hide()
 
+        self._gui.connect_signals(self)
 
     def load_config(self):
         self.get_widget("shutdown_track").set_active(self.config.get_stop_on_shutdown())
         self.get_widget("idle_track").set_active(self.config.get_timeout_enabled())
         self.get_widget("notify_interval").set_value(self.config.get_notify_interval())
         self.get_widget("keybinding").set_text(self.config.get_keybinding())
+        self.get_widget("notify_on_idle").set_active(self.config.get_notify_on_idle())
 
+        self.day_start.set_time(self.config.get_day_start())
+        
+        self.tags = [tag["name"] for tag in runtime.storage.get_tags(autocomplete=True)]
+        self.get_widget("autocomplete_tags").set_text(", ".join(self.tags))
+            
 
+    def on_autocomplete_tags_view_focus_out_event(self, view, event):
+        buf = self.get_widget("autocomplete_tags")
+        updated_tags = buf.get_text(buf.get_start_iter(), buf.get_end_iter(), 0) \
+                          .decode("utf-8")
+
+        if updated_tags == self.tags:
+            return
+        
+        runtime.storage.update_autocomplete_tags(updated_tags)
+        
+        
     def drag_data_get_data(self, treeview, context, selection, target_id,
                            etime):
         treeselection = treeview.get_selection()
@@ -253,13 +278,13 @@ class PreferencesEditor:
             iter = model.get_iter(path)
             if (position == gtk.TREE_VIEW_DROP_BEFORE
                 or position == gtk.TREE_VIEW_DROP_INTO_OR_BEFORE):
-                print "insert '%s' before '%s'" % (data, model[iter][3])
-                storage.move_activity(int(data), model[iter][3], insert_after = False)
+                logging.debug("insert '%s' before '%s'" % (data, model[iter][3]))
+                runtime.storage.move_activity(int(data), model[iter][3], insert_after = False)
             else:
-                print "insert '%s' after '%s'" % (data, model[iter][3])
-                storage.move_activity(int(data), model[iter][3], insert_after = True)
+                logging.debug("insert '%s' after '%s'" % (data, model[iter][3]))
+                runtime.storage.move_activity(int(data), model[iter][3], insert_after = True)
         else:
-            print "append '%s'" % data
+            logging.debug("append '%s'" % data)
 
         if context.action == gtk.gdk.ACTION_MOVE:
             context.finish(True, True, etime)
@@ -280,7 +305,7 @@ class PreferencesEditor:
         if drop_info:
             path, position = drop_info
             iter = model.get_iter(path)
-            changed = storage.change_category(int(data), model[iter][0])
+            changed = runtime.storage.change_category(int(data), model[iter][0])
             
             context.finish(changed, True, etime)
         else:
@@ -309,7 +334,7 @@ class PreferencesEditor:
             return False #ignoring unsorted category
 
         #look for dupes
-        categories = storage.get_category_list()
+        categories = runtime.storage.get_category_list()
         for category in categories:
             if category['name'].lower() == new_text.lower():
                 if id == -2: # that was a new category
@@ -318,10 +343,10 @@ class PreferencesEditor:
                 return False
 
         if id == -2: #new category
-            id = storage.add_category(new_text.decode("utf-8"))
+            id = runtime.storage.add_category(new_text.decode("utf-8"))
             model[path][0] = id
         else:
-            storage.update_category(id, new_text)
+            runtime.storage.update_category(id, new_text.decode("utf-8"))
 
         model[path][1] = new_text
 
@@ -331,7 +356,7 @@ class PreferencesEditor:
         category_id = model[path][2]
         
         #look for dupes
-        activities = storage.get_activities(category_id)
+        activities = runtime.storage.get_activities(category_id)
         for activity in activities:
             if activity['name'].lower() == new_text.lower():
                 if id == -1: # that was a new category
@@ -339,7 +364,11 @@ class PreferencesEditor:
                 self.select_activity(activity['id'])
                 return False
         
-        model[path][0] = storage.add_activity(new_text.decode("utf-8"), category_id)
+        
+        if id == -1: #new activity -> add
+            model[path][0] = runtime.storage.add_activity(new_text.decode("utf-8"), category_id)
+        else: #existing activity -> update
+            runtime.storage.update_activity(id, new_text.decode("utf-8"), category_id)
         model[path][1] = new_text
         return True
         
@@ -348,7 +377,7 @@ class PreferencesEditor:
         """ enables and disables action buttons depending on selected item """
         (model, iter) = selection.get_selected()
         id = 0
-        if iter == None:
+        if iter is None:
             self.activity_store.clear()
         else:
             self.prev_selected_activity = None
@@ -390,7 +419,7 @@ class PreferencesEditor:
             first_item = model.get_path(iter) == (0,)
             self.get_widget('activity_up').set_sensitive(not first_item)
 
-            last_item = model.iter_next(iter) == None
+            last_item = model.iter_next(iter) is None
             self.get_widget('activity_down').set_sensitive(not last_item)
 
     def _del_selected_row(self, tree):
@@ -486,7 +515,7 @@ class PreferencesEditor:
     def remove_current_activity(self):
         selection = self.activity_tree.get_selection()
         (model, iter) = selection.get_selected()
-        storage.remove_activity(model[iter][0])
+        runtime.storage.remove_activity(model[iter][0])
         self._del_selected_row(self.activity_tree)
 
 
@@ -525,7 +554,7 @@ class PreferencesEditor:
         (model, iter) = selection.get_selected()
         id = model[iter][0]
         if id != -1:
-            storage.remove_category(id)
+            runtime.storage.remove_category(id)
             self._del_selected_row(self.category_tree)
 
     def on_preferences_window_key_press(self, widget, event):
@@ -576,14 +605,14 @@ class PreferencesEditor:
 
     def on_activity_remove_clicked(self, button):
         removable_id = self._del_selected_row(self.activity_tree)
-        storage.remove_activity(removable_id)
+        runtime.storage.remove_activity(removable_id)
 
     def on_activity_up_clicked(self, button):
         (model, iter) = self.selection.get_selected()
 
         #previous item
         prev_iter = get_prev(self.selection, model)
-        storage.swap_activities(model[iter][0], model[iter][3],
+        runtime.storage.swap_activities(model[iter][0], model[iter][3],
                                 model[prev_iter][0], model[prev_iter][3])
         model.move_before(iter, prev_iter)
 
@@ -593,7 +622,7 @@ class PreferencesEditor:
         (model, iter) = self.selection.get_selected()
 
         next_iter = model.iter_next(iter)
-        storage.swap_activities(model[iter][0], model[iter][3],
+        runtime.storage.swap_activities(model[iter][0], model[iter][3],
                                 model[next_iter][0], model[next_iter][3])
         self.activity_store.move_after(iter, next_iter)
 
@@ -618,6 +647,9 @@ class PreferencesEditor:
     def on_idle_track_toggled(self, checkbox):
         self.config.set_timeout_enabled(checkbox.get_active())
 
+    def on_notify_on_idle_toggled(self, checkbox):
+        self.config.set_notify_on_idle(checkbox.get_active())
+
     def on_notify_interval_format_value(self, slider, value):
         if value <=120:
             # notify interval slider value label
@@ -629,11 +661,20 @@ class PreferencesEditor:
         return label
     
     def on_notify_interval_value_changed(self, scale):
-        self.config.set_notify_interval(int(scale.get_value()))
+        value = int(scale.get_value())
+        self.config.set_notify_interval(value)
+        self.get_widget("notify_on_idle").set_sensitive(value <= 120)
     
     def on_keybinding_changed(self, textbox):
         self.config.set_keybinding(textbox.get_text())
 
+    def on_day_start_changed(self, widget):
+        day_start = self.day_start.get_time()
+        if not day_start:
+            return
+
+        self.config.set_day_start(day_start)
+
     def on_preferences_window_destroy(self, window):
         self.window = None
         
