@@ -29,7 +29,10 @@ import stuff
 import charting
 
 from edit_activity import CustomFactController
-import reports, widgets, graphics
+import reports, graphics
+
+import widgets
+
 from configuration import runtime, GconfStore
 import webbrowser
 
@@ -41,293 +44,6 @@ import calendar
 import time
 from hamster.i18n import C_
 
-class ReportChooserDialog(gtk.Dialog):
-    __gsignals__ = {
-        # format, path, start_date, end_date
-        'report-chosen': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE,
-                          (gobject.TYPE_STRING, gobject.TYPE_STRING,
-                           gobject.TYPE_PYOBJECT, gobject.TYPE_PYOBJECT,
-                           gobject.TYPE_PYOBJECT)),
-        'report-chooser-closed': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, ()),
-    }
-    def __init__(self):
-        gtk.Dialog.__init__(self)
-        ui = stuff.load_ui_file("stats.ui")
-        self.dialog = ui.get_object('save_report_dialog')
-
-        self.dialog.set_action(gtk.FILE_CHOOSER_ACTION_SAVE)
-        self.dialog.set_current_folder(os.path.expanduser("~"))
-
-        self.filters = {}
-
-        filter = gtk.FileFilter()
-        filter.set_name(_("HTML Report"))
-        filter.add_mime_type("text/html")
-        filter.add_pattern("*.html")
-        filter.add_pattern("*.htm")
-        self.filters[filter] = "html"
-        self.dialog.add_filter(filter)
-
-        filter = gtk.FileFilter()
-        filter.set_name(_("Tab-Separated Values (TSV)"))
-        filter.add_mime_type("text/plain")
-        filter.add_pattern("*.tsv")
-        filter.add_pattern("*.txt")
-        self.filters[filter] = "tsv"
-        self.dialog.add_filter(filter)
-
-        filter = gtk.FileFilter()
-        filter.set_name(_("XML"))
-        filter.add_mime_type("text/xml")
-        filter.add_pattern("*.xml")
-        self.filters[filter] = "xml"
-        self.dialog.add_filter(filter)
-
-        filter = gtk.FileFilter()
-        filter.set_name(_("iCal"))
-        filter.add_mime_type("text/calendar")
-        filter.add_pattern("*.ics")
-        self.filters[filter] = "ical"
-        self.dialog.add_filter(filter)
-
-        filter = gtk.FileFilter()
-        filter.set_name("All files")
-        filter.add_pattern("*")
-        self.dialog.add_filter(filter)
-        
-        self.start_date = widgets.DateInput()
-        ui.get_object('from_date_box').add(self.start_date)
-        self.end_date = widgets.DateInput()
-        ui.get_object('to_date_box').add(self.end_date)
-
-        self.category_box = ui.get_object('category_box')
-
-        ui.get_object('save_button').connect("clicked", self.on_save_button_clicked)
-        ui.get_object('cancel_button').connect("clicked", self.on_cancel_button_clicked)
-        
-
-    def show(self, start_date, end_date):
-        #set suggested name to something readable, replace backslashes with dots
-        #so the name is valid in linux
-        filename = "Time track %s - %s" % (start_date.strftime("%x").replace("/", "."),
-                                           end_date.strftime("%x").replace("/", "."))
-        self.dialog.set_current_name(filename)
-        
-        self.start_date.set_date(start_date)
-        self.end_date.set_date(end_date)
-        
-        #add unsorted category
-        button_all = gtk.CheckButton(C_("categories", "All").encode("utf-8"))
-        button_all.value = None
-        button_all.set_active(True)
-        
-        def on_category_all_clicked(checkbox):
-            active = checkbox.get_active()
-            for checkbox in self.category_box.get_children():
-                checkbox.set_active(active)
-        
-        button_all.connect("clicked", on_category_all_clicked)
-        self.category_box.attach(button_all, 0, 1, 0, 1)
-
-        categories = runtime.storage.get_category_list()
-        col, row = 0, 0
-        for category in categories:
-            col +=1
-            if col % 4 == 0:
-                col = 0
-                row +=1
-
-            button = gtk.CheckButton(category['name'].encode("utf-8"))
-            button.value = category['id']
-            button.set_active(True)
-            self.category_box.attach(button, col, col+1, row, row+1)
-
-        
-
-        response = self.dialog.show_all()
-
-    def present(self):
-        self.dialog.present()
-
-    def on_save_button_clicked(self, widget):
-        path, format = None,  None
-
-        format = "html"
-        if self.dialog.get_filter() in self.filters:
-            format = self.filters[self.dialog.get_filter()]
-        path = self.dialog.get_filename()
-        
-        categories = []
-        for button in self.category_box.get_children():
-            if button.get_active():
-                categories.append(button.value)
-        
-        if None in categories:
-            categories = None # nothing is everything
-        
-        # format, path, start_date, end_date
-        self.emit("report-chosen", format, path,
-                           self.start_date.get_date().date(),
-                           self.end_date.get_date().date(),
-                           categories)
-        self.dialog.destroy()
-        
-
-    def on_cancel_button_clicked(self, widget):
-        self.emit("report-chooser-closed")
-        self.dialog.destroy()
-
-class TimeLine(graphics.Area):
-    MODE_YEAR = 0
-    MODE_MONTH = 1
-    MODE_WEEK = 1
-    MODE_DAY = 3
-    def __init__(self):
-        graphics.Area.__init__(self)
-        self.start_date, self.end_date = None, None
-        self.draw_mode = None
-        self.max_hours = None
-        
-    
-    def draw(self, facts):
-        import itertools
-        self.facts = {}
-        for date, date_facts in itertools.groupby(facts, lambda x: x["start_time"].date()):
-            date_facts = list(date_facts)
-            self.facts[date] = date_facts
-            self.max_hours = max(self.max_hours,
-                                 sum([fact["delta"].seconds / 60 / float(60) +
-                               fact["delta"].days * 24 for fact in date_facts]))
-        
-        start_date = facts[0]["start_time"].date()
-        end_date = facts[-1]["start_time"].date()
-
-        self.draw_mode = self.MODE_YEAR
-        self.start_date = start_date.replace(month=1, day=1)
-        self.end_date = end_date.replace(month=12, day=31)
-        
-
-        """
-        #TODO - for now we have only the year mode        
-        if start_date.year != end_date.year or start_date.month != end_date.month:
-            self.draw_mode = self.MODE_YEAR
-            self.start_date = start_date.replace(month=1, day=1)
-            self.end_date = end_date.replace(month=12, day=31)
-        elif start_date.strftime("%W") != end_date.strftime("%W"):
-            self.draw_mode = self.MODE_MONTH
-            self.start_date = start_date.replace(day=1)
-            self.end_date = end_date.replace(date =
-                                    calendar.monthrange(self.end_date.year,
-                                                        self.end_date.month)[1])
-        elif start_date != end_date:
-            self.draw_mode = self.MODE_WEEK
-        else:
-            self.draw_mode = self.MODE_DAY
-        """
-        
-        self.redraw_canvas()
-        
-        
-    def _render(self):
-        import calendar
-        
-        if self.draw_mode != self.MODE_YEAR:
-            return
-
-        self.fill_area(0, 0, self.width, self.height, (0.975,0.975,0.975))
-        self.set_color((100,100,100))
-
-        self.set_value_range(x_min = 1, x_max = (self.end_date - self.start_date).days)        
-        month_label_fits = True
-        for month in range(1, 13):
-            self.layout.set_text(calendar.month_abbr[month])
-            label_w, label_h = self.layout.get_pixel_size()
-            if label_w * 2 > self.x_factor * 30:
-                month_label_fits = False
-                break
-        
-        
-        ticker_date = self.start_date
-        
-        year_pos = 0
-        
-        for year in range(self.start_date.year, self.end_date.year + 1):
-            #due to how things lay over, we are putting labels on backwards, so that they don't overlap
-            
-            self.context.set_line_width(1)
-            for month in range(1, 13):
-                for day in range(1, calendar.monthrange(year, month)[1] + 1):
-                    ticker_pos = year_pos + ticker_date.timetuple().tm_yday
-                    
-                    #if ticker_date.weekday() in [0, 6]:
-                    #    self.fill_area(ticker_pos * self.x_factor + 1, 20, self.x_factor, self.height - 20, (240, 240, 240))
-                    #    self.context.stroke()
-                        
-    
-                    if self.x_factor > 5:
-                        self.move_to(ticker_pos, self.height - 20)
-                        self.line_to(ticker_pos, self.height)
-                   
-                        self.layout.set_text(ticker_date.strftime("%d"))
-                        label_w, label_h = self.layout.get_pixel_size()
-                        
-                        if label_w < self.x_factor / 1.2: #if label fits
-                            self.context.move_to(self.get_pixel(ticker_pos) + 2,
-                                                 self.height - 20)
-                            self.context.show_layout(self.layout)
-                    
-                        self.context.stroke()
-                        
-                    #now facts
-                    facts_today = self.facts.get(ticker_date, [])
-                    if facts_today:
-                        total_length = dt.timedelta()
-                        for fact in facts_today:
-                            total_length += fact["delta"]
-                        total_length = total_length.seconds / 60 / 60.0 + total_length.days * 24
-                        total_length = total_length / float(self.max_hours) * self.height - 16
-
-                        self.fill_area(round(ticker_pos * self.x_factor),
-                                       round(self.height - total_length),
-                                       round(self.x_factor),
-                                       round(total_length),
-                                       (190,190,190))
-
-
-                        
-
-                    ticker_date += dt.timedelta(1)
-                
-            
-                
-                if month_label_fits:
-                    #roll back a little
-                    month_pos = ticker_pos - calendar.monthrange(year, month)[1] + 1
-
-                    self.move_to(month_pos, 0)
-                    #self.line_to(month_pos, 20)
-                    
-                    self.layout.set_text(dt.date(year, month, 1).strftime("%b"))
-    
-                    self.move_to(month_pos, 0)
-                    self.context.show_layout(self.layout)
-
-
-    
-            
-    
-            self.layout.set_text("%d" % year)
-            label_w, label_h = self.layout.get_pixel_size()
-                        
-            self.move_to(year_pos + 2 / self.x_factor, month_label_fits * label_h * 1.2)
-    
-            self.context.show_layout(self.layout)
-            
-            self.context.stroke()
-
-            year_pos = ticker_pos #save current state for next year
-
-
 
 class StatsViewer(object):
     def __init__(self, parent = None):
@@ -414,7 +130,7 @@ class StatsViewer(object):
         self._gui.connect_signals(self)
         self.fact_tree.grab_focus()
 
-        self.timeline = TimeLine()
+        self.timeline = widgets.TimeLine()
         self.get_widget("explore_everything").add(self.timeline)
         self.get_widget("explore_everything").show_all()
 
@@ -500,7 +216,7 @@ class StatsViewer(object):
                 self.text = text
                 self.redraw_canvas()
                 
-            def _render(self):
+            def on_expose(self):
                 if self.background:
                     self.fill_area(0, 0, self.width, self.height, self.background)
 
@@ -874,8 +590,7 @@ than 15 minutes you seem to be a busy bee." % ("<b>%d</b>" % short_percent))
         #the single "totals" (by category) bar
         category_sums = stuff.totals(facts, lambda fact: fact["category"],
                       lambda fact: stuff.duration_minutes(fact["delta"]) / 60.0)
-        category_totals = [category_sums.get(cat, 0)
-                                                      for cat in all_categories]
+        category_totals = [category_sums.get(cat, 0) for cat in all_categories]
         category_keys = ["%s %.1f" % (cat, category_sums.get(cat, 0.0))
                                                       for cat in all_categories]
         self.category_chart.plot([_("Total")],
@@ -884,15 +599,16 @@ than 15 minutes you seem to be a busy bee." % ("<b>%d</b>" % short_percent))
         
         # day / category chart
         all_days = [self.start_date + dt.timedelta(i)
-                    for i in range((self.end_date - self.start_date).days  + 1)]        
-
+                    for i in range((self.end_date - self.start_date).days  + 1)]
+        
         by_date_cat = stuff.totals(facts,
-                                   lambda fact: (fact["date"],
-                                                 fact["category"]),
+                                   lambda fact: (fact["date"], fact["category"]),
                                    lambda fact: stuff.duration_minutes(fact["delta"]) / 60.0)
+
         res = [[by_date_cat.get((day, cat), 0)
                                  for cat in all_categories] for day in all_days]
 
+
         #show days or dates depending on scale
         if (self.end_date - self.start_date).days < 20:
             day_keys = [day.strftime("%a") for day in all_days]
@@ -911,15 +627,22 @@ than 15 minutes you seem to be a busy bee." % ("<b>%d</b>" % short_percent))
                                      lambda fact: (fact["name"],
                                                    fact["category"]),
                                      lambda fact: stuff.duration_minutes(fact["delta"]))
-        by_duration = sorted(activity_sums.items(),
-                             key = lambda x: x[1],
+        
+        #now join activities with same name
+        activities = {}
+        for key in activity_sums.keys():
+            activities.setdefault(key[0], [0.0] * len(all_categories))
+            activities[key[0]][all_categories.index(key[1])] = activity_sums[key] / 60.0
+            
+        by_duration = sorted(activities.items(),
+                             key = lambda x: sum(x[1]),
                              reverse = True)
-        by_duration_keys = [entry[0][0] for entry in by_duration]
+        by_duration_keys = [entry[0] for entry in by_duration]
+        
+        by_duration = [entry[1] for entry in by_duration]
 
-        category_sums = [[entry[1] / 60.0 * (entry[0][1] == cat)
-                            for cat in all_categories] for entry in by_duration]
         self.activity_chart.plot(by_duration_keys,
-                                 category_sums,
+                                 by_duration,
                                  stack_keys = all_categories)
         
 
@@ -1319,7 +1042,7 @@ than 15 minutes you seem to be a busy bee." % ("<b>%d</b>" % short_percent))
         
     def on_report_button_clicked(self, widget):
         if not self.report_chooser:
-            self.report_chooser = ReportChooserDialog()
+            self.report_chooser = widgets.ReportChooserDialog()
             self.report_chooser.connect("report-chosen", self.on_report_chosen)
             self.report_chooser.connect("report-chooser-closed",
                                         self.on_report_chooser_closed)
